import re
from typing import List, Union

import httpx
import pandas as pd
from pydantic import BaseModel

from slark.resources._resources import APIResource, AsyncAPIResource
from slark.types._utils import cached_property
from slark.types.bitables.record.response import RecordResponseData

from .field import AsyncField, Field
from .meta import AsyncMeta, Meta
from .record import AsyncRecord, Record
from .table import AsyncTable, Table
from .utils import dataframe_to_records, fields_records_to_dataframe
from .view import AsyncView, View


class BitableInfo(BaseModel):
    app_token: str
    table_id: str
    view_id: Union[str, None] = None


class AsyncBiTable(AsyncAPIResource):
    @cached_property
    def field(self) -> AsyncField:
        return AsyncField(self._client)

    @cached_property
    def record(self) -> AsyncRecord:
        return AsyncRecord(self._client)

    @cached_property
    def view(self) -> AsyncView:
        return AsyncView(self._client)

    @cached_property
    def table(self) -> AsyncTable:
        return AsyncTable(self._client)

    @cached_property
    def meta(self) -> AsyncMeta:
        return AsyncMeta(self._client)

    async def get_bitable_info(self, url: str) -> BitableInfo:
        """从多维表格分享链接中提取 app_token, table_id, view_id

        Args:
            url (str): URL of the bitable

        Returns:
            BitableInfo: app_token, table_id, view_id
        """
        if "/base/" in url:
            app_token, table_id, view_id = re.findall(
                r"\/base\/([^\/\?]+)(?:\?table=([^\/\&\?]+))?(?:\&view=([^\/]+))?", url
            )[0]
        elif "/wiki/" in url:
            wiki_token, table_id, view_id = re.findall(
                r"\/wiki\/([^\/\?]+)(?:\?table=([^\/\&\?]+))?(?:\&view=([^\/]+))?", url
            )[0]
            node_info = (
                await self._client.knowledge_space.nodes.get_node_info(wiki_token)
            ).data.node
            if node_info.obj_type != "bitable":
                raise ValueError("The node is not a bitable")
            app_token = node_info.obj_token
        else:
            raise ValueError("The url is not a bitable url")
        assert app_token, "BiTable token is not found"
        if not table_id:
            table_id = (await self.table.list(app_token)).data.items[0].table_id
        return BitableInfo(app_token=app_token, table_id=table_id, view_id=view_id)

    async def read(
        self,
        url: str,
        *,
        rows: Union[int, None] = None,
        field_names: Union[List[str], None] = None,
        return_raw: bool = False,
        timezone: Union[str, None] = "Asia/Shanghai",
        timeout: Union[httpx.Timeout, None] = None,
    ) -> Union[dict, pd.DataFrame]:
        """从多维表格中读取数据

        Args:
            url (str): 多维表格分享链接
            rows (Union[int, None], optional): 读取的行数. Defaults to None.
            field_names (Union[List[str], None], optional): 读取的列名. Defaults to None.
            raw (bool, optional): 是否返回原始数据. Defaults to False.
            timezone (Union[str, None], optional): 时区，仅在raw=False时有效. Defaults to "Asia/Shanghai".

        Returns:
            Union[dict, pd.DataFrame]: 当raw=True时返回原始数据，否则返回DataFrame\
                返回的 dataframe 的 index 为对应记录的 record id
        """
        info = await self.get_bitable_info(url)

        items: List[RecordResponseData] = []
        has_more: bool = True
        page_token: str = None
        if rows is None or rows > self.record.MAX_RECORDS_PER_REQUEST:
            page_size = self.record.MAX_RECORDS_PER_REQUEST
        else:
            page_size = rows

        def is_continue():
            return has_more and (rows is None or len(items) < rows)

        fields = (
            await self.field.list(app_token=info.app_token, table_id=info.table_id)
        ).data.items
        if field_names is not None:
            fields = [field for field in fields if field.field_name in field_names]
        while is_continue():
            response = await self.record.search(
                app_token=info.app_token,
                table_id=info.table_id,
                view_id=info.view_id,
                field_names=field_names,
                page_token=page_token,
                page_size=page_size,
                timeout=timeout,
            )
            has_more = response.data.has_more
            page_token = response.data.page_token
            items.extend(response.data.items)

        items = items[:rows] if rows else items
        if return_raw:
            return {"items": items, "fields": fields}
        else:
            return fields_records_to_dataframe(fields, items, timezone=timezone)

    async def append(
        self,
        url: str,
        *,
        data: pd.DataFrame,
        timezone: Union[str, None] = "Asia/Shanghai",
        timeout: Union[httpx.Timeout, None] = None,
    ) -> List[RecordResponseData]:
        """向多维表格中追加数据

        Args:
            url (str): 多维表格分享链接
            data (pd.DataFrame): 要追加的数据
            timezone (Union[str, None], optional): 时区. Defaults to "Asia/Shanghai".
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            List[RecordResponseData]: 追加的数据
        """
        info = await self.get_bitable_info(url)
        records = dataframe_to_records(data, timezone=timezone)
        response_records: List[RecordResponseData] = []
        for i in range(0, len(records), self.record.MAX_RECORDS_PER_REQUEST):
            response = await self.record.batch_create(
                app_token=info.app_token,
                table_id=info.table_id,
                records=records[i : i + self.record.MAX_RECORDS_PER_REQUEST],
                timeout=timeout,
            )
            response_records.extend(response.data.records)
        return response_records

    async def update(
        self,
        url: str,
        *,
        data: pd.DataFrame,
        timezone: Union[str, None] = "Asia/Shanghai",
        timeout: Union[httpx.Timeout, None] = None,
    ):
        """更新多维表格中的数据

        Args:
            url (str): 多维表格分享链接
            data (pd.DataFrame): 要更新的数据，index 为更新记录的 record id
            timezone (Union[str, None], optional): 时区. Defaults to "Asia/Shanghai".
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            List[RecordResponseData]: 更新的数据
        """
        info = await self.get_bitable_info(url)
        records = dataframe_to_records(data, timezone=timezone, use_index_as_record_id=True)
        response_records: List[RecordResponseData] = []
        for i in range(0, len(records), self.record.MAX_RECORDS_PER_REQUEST):
            response = await self.record.batch_update(
                app_token=info.app_token,
                table_id=info.table_id,
                records=records[i : i + self.record.MAX_RECORDS_PER_REQUEST],
                timeout=timeout,
            )
            response_records.extend(response.data.records)
        return response_records

    async def delete(
        self, url: str, *, record_ids: List[str], timeout: Union[httpx.Timeout, None] = None
    ) -> None:
        """删除多维表格中的数据

        Args:
            url (str): 多维表格分享链接
            record_ids (List[str]): 要删除的记录ID
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.
        """
        info = await self.get_bitable_info(url)
        delete_response = []
        for i in range(0, len(record_ids), self.record.MAX_RECORDS_PER_REQUEST):
            response = await self.record.batch_delete(
                app_token=info.app_token,
                table_id=info.table_id,
                record_ids=record_ids[i : i + self.record.MAX_RECORDS_PER_REQUEST],
                timeout=timeout,
            )
            delete_response.extend(response.data.records)
        return delete_response


class BiTable(APIResource):
    @cached_property
    def field(self) -> Field:
        return Field(self._client)

    @cached_property
    def record(self) -> Record:
        return Record(self._client)

    @cached_property
    def view(self) -> View:
        return View(self._client)

    @cached_property
    def table(self) -> Table:
        return Table(self._client)

    @cached_property
    def meta(self) -> Meta:
        return Meta(self._client)

    def get_bitable_info(self, url: str) -> BitableInfo:
        """从多维表格分享链接中提取 app_token, table_id, view_id

        Args:
            url (str): URL of the bitable

        Returns:
            BitableInfo: app_token, table_id, view_id
        """
        if "/base/" in url:
            app_token, table_id, view_id = re.findall(
                r"\/base\/([^\/\?]+)(?:\?table=([^\/\&\?]+))?(?:\&view=([^\/]+))?", url
            )[0]
        elif "/wiki/" in url:
            wiki_token, table_id, view_id = re.findall(
                r"\/wiki\/([^\/\?]+)(?:\?table=([^\/\&\?]+))?(?:\&view=([^\/]+))?", url
            )[0]
            node_info = (self._client.knowledge_space.nodes.get_node_info(wiki_token)).data.node
            if node_info.obj_type != "bitable":
                raise ValueError("The node is not a bitable")
            app_token = node_info.obj_token
        else:
            raise ValueError("The url is not a bitable url")
        assert app_token, "BiTable token is not found"
        if not table_id:
            table_id = (self.table.list(app_token)).data.items[0].table_id
        return BitableInfo(app_token=app_token, table_id=table_id, view_id=view_id)

    def read(
        self,
        url: str,
        *,
        rows: Union[int, None] = None,
        field_names: Union[List[str], None] = None,
        return_raw: bool = False,
        timezone: Union[str, None] = "Asia/Shanghai",
        timeout: Union[httpx.Timeout, None] = None,
    ) -> Union[dict, pd.DataFrame]:
        """从多维表格中读取数据

        Args:
            url (str): 多维表格分享链接
            rows (Union[int, None], optional): 读取的行数. Defaults to None.
            field_names (Union[List[str], None], optional): 读取的列名. Defaults to None.
            raw (bool, optional): 是否返回原始数据. Defaults to False.
            timezone (Union[str, None], optional): 时区，仅在raw=False时有效. Defaults to "Asia/Shanghai".

        Returns:
            Union[dict, pd.DataFrame]: 当raw=True时返回原始数据，否则返回DataFrame\
                返回的 dataframe 的 index 为对应记录的 record id
        """
        info = self.get_bitable_info(url)

        items: List[RecordResponseData] = []
        has_more: bool = True
        page_token: str = None
        if rows is None or rows > self.record.MAX_RECORDS_PER_REQUEST:
            page_size = self.record.MAX_RECORDS_PER_REQUEST
        else:
            page_size = rows

        def is_continue():
            return has_more and (rows is None or len(items) < rows)

        fields = (self.field.list(app_token=info.app_token, table_id=info.table_id)).data.items
        if field_names is not None:
            fields = [field for field in fields if field.field_name in field_names]
        while is_continue():
            response = self.record.search(
                app_token=info.app_token,
                table_id=info.table_id,
                view_id=info.view_id,
                field_names=field_names,
                page_token=page_token,
                page_size=page_size,
                timeout=timeout,
            )
            has_more = response.data.has_more
            page_token = response.data.page_token
            items.extend(response.data.items)

        items = items[:rows] if rows else items
        if return_raw:
            return {"items": items, "fields": fields}
        else:
            return fields_records_to_dataframe(fields, items, timezone=timezone)

    def append(
        self,
        url: str,
        *,
        data: pd.DataFrame,
        timezone: Union[str, None] = "Asia/Shanghai",
        timeout: Union[httpx.Timeout, None] = None,
    ) -> List[RecordResponseData]:
        """向多维表格中追加数据

        Args:
            url (str): 多维表格分享链接
            data (pd.DataFrame): 要追加的数据
            timezone (Union[str, None], optional): 时区. Defaults to "Asia/Shanghai".
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            List[RecordResponseData]: 追加的数据
        """
        info = self.get_bitable_info(url)
        records = dataframe_to_records(data, timezone=timezone)
        response_records: List[RecordResponseData] = []
        for i in range(0, len(records), self.record.MAX_RECORDS_PER_REQUEST):
            response = self.record.batch_create(
                app_token=info.app_token,
                table_id=info.table_id,
                records=records[i : i + self.record.MAX_RECORDS_PER_REQUEST],
                timeout=timeout,
            )
            response_records.extend(response.data.records)
        return response_records

    def update(
        self,
        url: str,
        *,
        data: pd.DataFrame,
        timezone: Union[str, None] = "Asia/Shanghai",
        timeout: Union[httpx.Timeout, None] = None,
    ):
        """更新多维表格中的数据

        Args:
            url (str): 多维表格分享链接
            data (pd.DataFrame): 要更新的数据，index 为更新记录的 record id
            timezone (Union[str, None], optional): 时区. Defaults to "Asia/Shanghai".
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            List[RecordResponseData]: 更新的数据
        """
        info = self.get_bitable_info(url)
        records = dataframe_to_records(data, timezone=timezone, use_index_as_record_id=True)
        response_records: List[RecordResponseData] = []
        for i in range(0, len(records), self.record.MAX_RECORDS_PER_REQUEST):
            response = self.record.batch_update(
                app_token=info.app_token,
                table_id=info.table_id,
                records=records[i : i + self.record.MAX_RECORDS_PER_REQUEST],
                timeout=timeout,
            )
            response_records.extend(response.data.records)
        return response_records

    def delete(
        self, url: str, *, record_ids: List[str], timeout: Union[httpx.Timeout, None] = None
    ) -> None:
        """删除多维表格中的数据

        Args:
            url (str): 多维表格分享链接
            record_ids (List[str]): 要删除的记录ID
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.
        """
        info = self.get_bitable_info(url)
        delete_response = []
        for i in range(0, len(record_ids), self.record.MAX_RECORDS_PER_REQUEST):
            response = self.record.batch_delete(
                app_token=info.app_token,
                table_id=info.table_id,
                record_ids=record_ids[i : i + self.record.MAX_RECORDS_PER_REQUEST],
                timeout=timeout,
            )
            delete_response.extend(response.data.records)
        return delete_response



from typing import final


@final
class KNOWLEDGE_SPACE_NODE:
    get_node = "/wiki/v2/spaces/get_node"


@final
class KNOWLEDGE_SPACE:
    nodes = KNOWLEDGE_SPACE_NODE


@final
class AUTH:
    get_tenant_access_token = "/auth/v3/tenant_access_token/internal"
    get_app_access_token = "/auth/v3/app_access_token/internal"
    get_user_access_token = "/authen/v1/oidc/access_token"


@final
class WORKSHEET_DATA:
    prepend_data = "/sheets/v2/spreadsheets/{spreadsheetToken}/values_prepend"
    append_data = "/sheets/v2/spreadsheets/{spreadsheetToken}/values_append"
    read_single_range_data = "/sheets/v2/spreadsheets/{spreadsheetToken}/values/{range}"
    read_multi_range_data = "/sheets/v2/spreadsheets/{spreadsheetToken}/values_batch_get"
    write_single_range_data = "/sheets/v2/spreadsheets/{spreadsheetToken}/values"
    write_multi_range_data = "/sheets/v2/spreadsheets/{spreadsheetToken}/values_batch_update"


@final
class SPREADSHEETS:
    get_spreadsheet_info = "/sheets/v3/spreadsheets/{spreadsheet_token}"
    get_all_worksheets = "/sheets/v3/spreadsheets/{spreadsheet_token}/sheets/query"
    get_worksheet_info = "/sheets/v3/spreadsheets/{spreadsheet_token}/sheets/{sheet_id}"
    data = WORKSHEET_DATA


@final
class BITABLES:
    get_bitable_meta = "/bitable/v1/apps/{app_token}"
    update_bitable_meta = "/bitable/v1/apps/{app_token}"

    create_table = "/bitable/v1/apps/{app_token}/tables"
    batch_create_table = "/bitable/v1/apps/{app_token}/tables/batch_create"
    delete_table = "/bitable/v1/apps/{app_token}/tables/{table_id}"
    batch_delete_table = "/bitable/v1/apps/{app_token}/tables/batch_delete"
    update_table = "/bitable/v1/apps/{app_token}/tables/{table_id}"
    list_tables = "/bitable/v1/apps/{app_token}/tables"

    copy_dashboard = "/bitable/v1/apps/{app_token}/dashboards/{block_id}/copy"
    list_dashboards = "/bitable/v1/apps/{app_token}/dashboards"

    update_view = "/bitable/v1/apps/{app_token}/tables/{table_id}/views/{view_id}"
    get_view_info = "/bitable/v1/apps/{app_token}/tables/{table_id}/views/{view_id}"
    list_views = "/bitable/v1/apps/{app_token}/tables/{table_id}/views"
    create_view = "/bitable/v1/apps/{app_token}/tables/{table_id}/views"
    delete_view = "/bitable/v1/apps/{app_token}/tables/{table_id}/views/{view_id}"

    update_form_meta = "/bitable/v1/apps/{app_token}/tables/{table_id}/forms/{form_id}"
    get_form_meta = "/bitable/v1/apps/{app_token}/tables/{table_id}/forms/{form_id}"
    update_form_field = (
        "/bitable/v1/apps/{app_token}/tables/{table_id}/forms/{form_id}/fields/{field_id}"
    )
    list_form_fileds = "/bitable/v1/apps/{app_token}/tables/{table_id}/forms/{form_id}/fields"

    create_record = "/bitable/v1/apps/{app_token}/tables/{table_id}/records"
    update_record = "/bitable/v1/apps/{app_token}/tables/{table_id}/records/{record_id}"
    search_record = "/bitable/v1/apps/{app_token}/tables/{table_id}/records/search"

    get_record = "/bitable/v1/apps/{app_token}/tables/{table_id}/records/{record_id}"
    delete_record = "/bitable/v1/apps/{app_token}/tables/{table_id}/records/{record_id}"

    batch_create_record = "/bitable/v1/apps/{app_token}/tables/{table_id}/records/batch_create"
    batch_update_record = "/bitable/v1/apps/{app_token}/tables/{table_id}/records/batch_update"
    batch_get_record = "/bitable/v1/apps/{app_token}/tables/{table_id}/records/batch_get"
    batch_delete_record = "/bitable/v1/apps/{app_token}/tables/{table_id}/records/batch_delete"

    list_field = "/bitable/v1/apps/{app_token}/tables/{table_id}/fields"
    create_field = "/bitable/v1/apps/{app_token}/tables/{table_id}/fields"
    update_field = "/bitable/v1/apps/{app_token}/tables/{table_id}/fields/{field_id}"
    delete_field = "/bitable/v1/apps/{app_token}/tables/{table_id}/fields/{field_id}"


@final
class DOCUMENTS:
    get_base_info = "/docx/v1/documents/{document_id}"
    get_raw_content = "/docx/v1/documents/{document_id}/raw_content"
    get_blocks = "/docx/v1/documents/{document_id}/blocks"
    create_document = "/docx/v1/documents"

    get_block_content = "/docx/v1/documents/{document_id}/blocks/{block_id}"
    get_children_blocks = "/docx/v1/documents/{document_id}/blocks/{block_id}/children"
    create_block = "/docx/v1/documents/{document_id}/blocks/{block_id}/children"
    update_block = "/docx/v1/documents/{document_id}/blocks/{block_id}"
    batch_update_block = "/docx/v1/documents/{document_id}/blocks/batch_update"
    batch_delete_block = "/docx/v1/documents/{document_id}/blocks/{block_id}/children/batch_delete"


@final
class BOARD:
    download_as_image = "/board/v1/whiteboards/{whiteboard_id}/download_as_image"


@final
class ASSETS:
    download = "/drive/v1/medias/{file_token}/download"


@final
class IMAGE:
    upload = "/im/v1/images"


@final
class MESSAGE:
    send = "im/v1/messages"
    reply = "im/v1/messages/{message_id}/reply"
    edit = "im/v1/messages/{message_id}"
    forward = "im/v1/messages/{message_id}/forward"
    get = "im/v1/messages/{message_id}"
    get_resource = "im/v1/messages/{message_id}/resources/{file_key}"


@final
class DRIVE_FILE:
    download = "drive/v1/files/{file_token}/download"


@final
class DRIVE:
    file = DRIVE_FILE


@final
class API_PATH:
    knowledge_space = KNOWLEDGE_SPACE
    auth = AUTH
    spreadsheets = SPREADSHEETS
    bitables = BITABLES
    documents = DOCUMENTS
    board = BOARD
    assets = ASSETS
    image = IMAGE
    message = MESSAGE
    drive = DRIVE



from typing_extensions import Literal

from slark.types._common import BaseModel
from slark.types.response.base import BaseResponse


class GetNodeResponseDataNode(BaseModel):
    space_id: str
    """知识空间id 获取方式 https://open.feishu.cn/document/server-docs/docs/wiki-v2/wiki-overview"""
    node_token: str
    """节点token"""
    obj_token: str
    """对应文档类型的token，可根据 obj_type 判断属于哪种文档类型。"""
    obj_type: Literal[
        "doc",
        "docx",
        "sheet",
        "mindnote",
        "bitable",
        "file",
        "slides",
    ]
    """文档类型，对于快捷方式，该字段是对应的实体的obj_type。
        
       可选值有：
       doc：旧版文档
       sheet：表格
       mindnote：思维导图
       bitable：多维表格
       file：文件
       docx：新版文档
       slides：幻灯片"""
    parent_node_token: str
    """父节点 token。若当前节点为一级节点，父节点 token 为空。"""
    node_type: Literal["origin", "shortcut"]
    """节点类型
    
       可选值有：
       origin：实体
       shortcut：快捷方式"""
    origin_node_token: str
    """快捷方式对应的实体node_token，当节点为快捷方式时，该值不为空。"""
    origin_space_id: str
    """	快捷方式对应的实体所在的space id"""
    has_child: bool
    """是否有子节点"""
    title: str
    """文档标题"""
    obj_create_time: str
    """文档创建时间"""
    obj_edit_time: str
    """文档最近编辑时间"""
    node_create_time: str
    """节点创建时间"""
    creator: str
    """文档创建者"""
    owner: str
    """文档所有者"""
    node_creator: str
    """节点创建者"""


class GetNodeResponseData(BaseModel):
    node: GetNodeResponseDataNode


class GetNodeResponse(BaseResponse):
    """获取节点详情返回数据
    https://open.feishu.cn/document/server-docs/docs/wiki-v2/space-node/get_node"""

    data: GetNodeResponseData



from slark.types._utils import cached_property

from .._resources import AsyncAPIResource
from .token import AsyncToken, Token


class AsyncAuth(AsyncAPIResource):
    @cached_property
    def token(self) -> AsyncToken:
        return AsyncToken(client=self._client)


class Auth(AsyncAPIResource):
    @cached_property
    def token(self) -> Token:
        return Token(client=self._client)



from typing import Union

from typing_extensions import Literal

from ..base import BaseElement
from ..color import Color
from ..icon import IconElement
from ..text import TextSize


class PlainTextElement(BaseElement):
    tag: Literal["plain_text", "lark_md"] = "plain_text"
    """文本类型的标签。可取值：

    plain_text：普通文本内容
    lark_md：支持部分 Markdown 语法的文本内容。详情参考下文 lark_md 支持的 Markdown 语法
    注意：飞书卡片搭建工具中仅支持使用 plain_text 类型的普通文本组件。你可使用富文本组件添加 Markdown 格式的文本。"""

    content: str
    """文本内容。当 tag 为 lark_md 时，支持部分 Markdown 语法的文本内容。详情参考下文 lark_md 支持的 Markdown 语法。
    https://open.feishu.cn/document/uAjLw4CM/ukzMukzMukzM/feishu-cards/card-components/content-components/plain-text#39ee4e65"""

    text_size: Union[TextSize, str, None] = TextSize.NORMAL.value
    """文本大小。可取值如下所示。如果你填写了其它值，卡片将展示为 normal 字段对应的字号。你也可分别为移动端和桌面端定义不同的字号，详细步骤参考下文 为移动端和桌面端定义不同的字号。"""
    text_align: Literal["left", "center", "right"] = "left"
    """文本对齐方式。可取值：

    left：左对齐
    center：居中对齐
    right：右对齐"""

    text_color: Union[str, Color] = "default"
    """文本的颜色。仅在 tag 为 plain_text 时生效。可取值：

    default：客户端浅色主题模式下为黑色；客户端深色主题模式下为白色"""

    lines: Union[int, None] = None
    """内容最大显示行数，超出设置行的内容用 ... 省略。"""

class DivElement(BaseElement):
    tag: str = "div"
    text: PlainTextElement
    icon: Union[IconElement, None] = None


from typing import List, Union

from typing_extensions import Literal

from ..base import BaseElement, BaseModel


class ImageCombinationList(BaseModel):
    img_key: str


class ImageCombinationElement(BaseElement):
    tag: str = "img_combination"
    combination_mode: Literal["double", "triple", "bisect", "trisect"] = "double"
    corner_radius: Union[str, None] = None
    img_list: List[ImageCombinationList]


from typing import Union

import httpx

from slark.resources._resources import APIResource, AsyncAPIResource
from slark.resources.api_path import API_PATH
from slark.types.knowledge_space.nodes.request import GetNodeQuery, NodeTypes
from slark.types.knowledge_space.nodes.response import GetNodeResponse


class AsyncNodes(AsyncAPIResource):
    async def get_node_info(
        self,
        token: str,
        obj_type: NodeTypes = "wiki",
        timeout: Union[httpx.Timeout, None] = None,
    ) -> GetNodeResponse:
        return await self._get(
            API_PATH.knowledge_space.nodes.get_node,
            cast_to=GetNodeResponse,
            options={
                "timeout": timeout,
                "params": GetNodeQuery(token=token, obj_type=obj_type).model_dump(),
            },
        )


class Nodes(APIResource):
    def get_node_info(
        self,
        token: str,
        obj_type: NodeTypes = "wiki",
        timeout: Union[httpx.Timeout, None] = None,
    ) -> GetNodeResponse:
        return self._get(
            API_PATH.knowledge_space.nodes.get_node,
            cast_to=GetNodeResponse,
            options={
                "timeout": timeout,
                "params": GetNodeQuery(token=token, obj_type=obj_type).model_dump(),
            },
        )



from typing import List, Union

from ..base import BaseElement
from ..color import Color


class CollapsibleElement(BaseElement):
    tag: str = "collapsible_panel"
    expanded: Union[bool, None] = None
    background_color: Union[Color, None] = None
    elements: List[BaseElement]


from pydantic import BaseModel

from slark.types.response import BaseResponse


class FileInfo(BaseModel):
    file_path: str
    file_token: str
    file_name: str
    mime_type: str
    

class FileDownloadResponse(BaseResponse):
    data: FileInfo


from .decode import extract_filename
from .thread import run_in_thread
from .time import datetime_now
from .unit import (
    dataframe_to_values,
    values_to_dataframe,
)

__all__ = [
    "datetime_now",
    "values_to_dataframe",
    "dataframe_to_values",
    "run_in_thread",
    "extract_filename"
]



from typing import Dict, List, Union

from slark.types._common import BaseModel
from slark.types.response import BaseResponse

from .common import FieldValueType, User


class RecordResponseData(BaseModel):
    record_id: str
    """一条记录的唯一标识 id"""
    created_by: Union[User, None] = None
    """该记录的创建人"""
    created_time: Union[int, None] = None
    """该记录的创建时间"""
    last_modified_by: Union[User, None] = None
    """该记录最新一次更新的修改人"""
    last_modified_time: Union[int, None] = None
    """该记录最近一次的更新时间"""
    fields: Dict[str, FieldValueType]
    """数据表的字段，即数据表的列"""


class CreateRecordResponseData(BaseModel):
    record: RecordResponseData


class CreateRecordResponse(BaseResponse):
    """创建记录相应体
        Example:
        ```
        {
        "code": 0,
        "data": {
            "record": {
                "fields": {
                    "条码": "+$$3170930509104X512356",
                    "人员": [
                        {
                            "id": "ou_2910013f1e6456f16a0ce75ede950a0a"
                        },
                        {
                            "id": "ou_e04138c9633dd0d2ea166d79f548ab5d"
                        }
                    ],
                    "单向关联": [
                        "recHTLvO7x",
                        "recbS8zb2m"
                    ],
                    "单选": "选项3",
                    "双向关联": [
                        "recHTLvO7x",
                        "recbS8zb2m"
                    ],
                    "地理位置": "116.397755,39.903179",
                    "复选框": true,
                    "多行文本": "多行文本内容",
                    "多选": [
                        "选项1",
                        "选项2"
                    ],
                    "数字": 100,
                    "日期": 1674206443000,
                    "电话号码": "13026162666",
                    "索引": "索引列多行文本类型",
                    "超链接": {
                        "link": "https://www.feishu.cn/product/base",
                        "text": "飞书多维表格官网"
                    },
                    "附件": [
                        {
                            "file_token": "Vl3FbVkvnowlgpxpqsAbBrtFcrd"
                        }
                    ],
                    "群组": [
                        {
                            "id": "oc_cd07f55f14d6f4a4f1b51504e7e97f48"
                        }
                    ],
                    "评分": 3,
                    "货币": 3,
                    "进度": 0.25
                },
                "id": "reclAqylTN",
                "record_id": "reclAqylTN"
            }
        },
        "msg": "success"
    }
    ```
    """

    data: CreateRecordResponseData


class UpdateRecordResponse(CreateRecordResponse):
    pass


class SearchRecordResponseData(BaseModel):
    items: List[RecordResponseData]
    """数组类型。record 结果。了解 record 数据结构，
    [参考数据结构](https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/bitable/development-guide/bitable-structure)"""

    has_more: bool
    """是否还有更多项"""

    page_token: Union[str, None] = None
    """分页标记，当 has_more 为 true 时，会同时返回新的 page_token，否则不返回 page_token"""

    total: int
    """总数"""


class SearchRecordResponse(BaseResponse):
    """搜索记录响应体
        Example:
        ```
        {
        "code":0,
        "data":{
            "has_more":false,
            "items":[
                {
                    "created_by":{
                        "avatar_url":"https://internal-api-lark-file.feishu.cn/static-resource/v1/06d568cb-f464-4c2e-bd03-76512c545c5j~?image_size=72x72&amp;cut_type=default-face&amp;quality=&amp;format=jpeg&amp;sticker_format=.webp",
                        "email":"",
                        "en_name":"测试1",
                        "id":"ou_92945f86a98bba075174776959c90eda",
                        "name":"测试1"
                    },
                    "created_time":1691049973000,
                    "fields":{
                        "人员":[
                            {
                                "avatar_url":"https://internal-api-lark-file.feishu.cn/static-resource/v1/b2-7619-4b8a-b27b-c72d90b06a2j~?image_size=72x72&amp;cut_type=default-face&amp;quality=&amp;format=jpeg&amp;sticker_format=.webp",
                                "email":"zhangsan.leben@bytedance.com",
                                "en_name":"ZhangSan",
                                "id":"ou_2910013f1e6456f16a0ce75ede950a0a",
                                "name":"张三"
                            },
                            {
                                "avatar_url":"https://internal-api-lark-file.feishu.cn/static-resource/v1/v2_q86-fcb6-4f18-85c7-87ca8881e50j~?image_size=72x72&amp;cut_type=default-face&amp;quality=&amp;format=jpeg&amp;sticker_format=.webp",
                                "email":"lisi.00@bytedance.com",
                                "en_name":"LiSi",
                                "id":"ou_e04138c9633dd0d2ea166d79f548ab5d",
                                "name":"李四"
                            }
                        ],
                        "修改人":[
                            {
                                "avatar_url":"https://internal-api-lark-file.feishu.cn/static-resource/v1/06d568cb-f464-4c2e-bd03-76512c545c5j~?image_size=72x72&amp;cut_type=default-face&amp;quality=&amp;format=jpeg&amp;sticker_format=.webp",
                                "email":"",
                                "en_name":"测试1",
                                "id":"ou_92945f86a98bba075174776959c90eda",
                                "name":"测试1"
                            }
                        ],
                        "创建人":[
                            {
                                "avatar_url":"https://internal-api-lark-file.feishu.cn/static-resource/v1/06d568cb-f464-4c2e-bd03-76512c545c5j~?image_size=72x72&amp;cut_type=default-face&amp;quality=&amp;format=jpeg&amp;sticker_format=.webp",
                                "email":"",
                                "en_name":"测试1",
                                "id":"ou_92945f86a98bba075174776959c90eda",
                                "name":"测试1"
                            }
                        ],
                        "创建时间":1691049973000,
                        "单向关联":{
                            "link_record_ids":[
                                "recnVYsuqV"
                            ]
                        },
                        "单选":"选项1",
                        "双向关联":{
                            "link_record_ids":[
                                "recqLvMaXT",
                                "recrdld32q"
                            ]
                        },
                        "地理位置":{
                            "address":"东长安街",
                            "adname":"东城区",
                            "cityname":"北京市",
                            "full_address":"天安门广场，北京市东城区东长安街",
                            "location":"116.397755,39.903179",
                            "name":"天安门广场",
                            "pname":"北京市"
                        },
                        "复选框":true,
                        "多行文本":[
                            {
                                "text":"多行文本内容1",
                                "type":"text"
                            },
                            {
                                "mentionNotify":false,
                                "mentionType":"User",
                                "name":"张三",
                                "text":"@张三",
                                "token":"ou_2910013f1e6456f16a0ce75ede950a0a",
                                "type":"mention"
                            }
                        ],
                        "多选":[
                            "选项1",
                            "选项2"
                        ],
                        "数字":2323.2323,
                        "日期":1690992000000,
                        "最后更新时间":1702455191000,
                        "条码":[
                            {
                                "text":"123",
                                "type":"text"
                            }
                        ],
                        "电话号码":"131xxxx6666",
                        "自动编号":"17",
                        "群组":[
                            {
                                "avatar_url":"https://internal-api-lark-file.feishu-boe.cn/static-resource/v1/v2_c8d2cd50-ba29-476f-b7f1-5b5917cb18ej~?image_size=72x72&amp;cut_type=&amp;quality=&amp;format=jpeg&amp;sticker_format=.webp",
                                "id":"oc_cd07f55f14d6f4a4f1b51504e7e97f48",
                                "name":"武侠聊天组"
                            }
                        ],
                        "评分":3,
                        "货币":1,
                        "超链接":{
                            "link":"https://bitable.feishu.cn",
                            "text":"飞书多维表格官网"
                        },
                        "进度":0.66,
                        "附件":[
                            {
                                "file_token":"Vl3FbVkvnowlgpxpqsAbBrtFcrd",
                                "name":"飞书.jpeg",
                                "size":32975,
                                "tmp_url":"https://open.feishu.cn/open-apis/drive/v1/medias/batch_get_tmp_download_url?file_tokens=Vl3FbVk11owlgpxpqsAbBrtFcrd&amp;extra={"bitablePerm":{"tableId":"tblBJyX6jZteblYv","rev":90}}",
                                "type":"image/jpeg",
                                "url":"https://open.feishu.cn/open-apis/drive/v1/medias/Vl3FbVk11owlgpxpqsAbBrtFcrd/download?extra={"bitablePerm":{"tableId":"tblBJyX6jZteblYv","rev":90}}"
                            }
                        ]
                    },
                    "last_modified_by":{
                        "avatar_url":"https://internal-api-lark-file.feishu.cn/static-resource/v1/06d568cb-f464-4c2e-bd03-76512c545c5j~?image_size=72x72&amp;cut_type=default-face&amp;quality=&amp;format=jpeg&amp;sticker_format=.webp",
                        "email":"",
                        "en_name":"测试1",
                        "id":"ou_92945f86a98bba075174776959c90eda",
                        "name":"测试1"
                    },
                    "last_modified_time":1702455191000,
                    "record_id":"recyOaMB2F"
                }
            ],
            "total":1
        },
        "msg":"success"
    }```
    """

    data: SearchRecordResponseData


class DeleteRecordResponseData(BaseModel):
    deleted: bool
    """是否成功删除"""
    record_id: str
    """删除的记录 ID"""


class DeleteRecordResponse(BaseResponse):
    """删除记录响应体
    Example:
    ```
    {"code": 0, "msg": "success", "data": {"deleted": true, "record_id": "recpCsf4ME"}}
    ```"""

    data: DeleteRecordResponseData


class BatchDeleteRecordResponseData(BaseModel):
    records: List[DeleteRecordResponseData]


class BatchDeleteRecordResponse(BaseResponse):
    """批量删除记录响应体
    Example:
    ```
    {
        "code": 0,
        "msg": "success",
        "data": {"records": [{"deleted": true, "record_id": "recpCsf4ME"}]},
    }
    ```"""

    data: BatchDeleteRecordResponseData


class BatchCreateRecordResponseData(BaseModel):
    records: List[RecordResponseData]


class BatchCreateRecordResponse(BaseResponse):
    """批量创建记录响应体
    Example:
    ```
    {
        "code": 0,
        "data": {
            "records": [
                {
                    "fields": {
                        "人员": [
                            {"id": "ou_2910013f1e6456f16a0ce75ede950a0a"},
                            {"id": "ou_e04138c9633dd0d2ea166d79f548ab5d"},
                        ],
                        "群组": [{"id": "oc_cd07f55f14d6f4a4f1b51504e7e97f48"}],
                        "单向关联": ["recHTLvO7x", "recbS8zb2m"],
                        "单选": "选项3",
                        "双向关联": ["recHTLvO7x", "recbS8zb2m"],
                        "地理位置": "116.397755,39.903179",
                        "复选框": true,
                        "多行文本": "多行文本内容",
                        "多选": ["选项1", "选项2"],
                        "数字": 100,
                        "日期": 1674206443000,
                        "条码": "qawqe",
                        "电话号码": "13026162666",
                        "索引": "索引列多行文本类型",
                        "超链接": {
                            "link": "https://www.feishu.cn/product/base",
                            "text": "飞书多维表格官网",
                        },
                        "附件": [{"file_token": "Vl3FbVkvnowlgpxpqsAbBrtFcrd"}],
                        "评分": 3,
                        "货币": 3,
                        "进度": 0.25,
                    },
                    "id": "recB8s1jUB",
                    "record_id": "recB8s1jUB",
                }
            ]
        },
        "msg": "success",
    }
    ```"""

    data: BatchCreateRecordResponseData


class BatchGetRecordResponseData(BaseModel):
    records: List[RecordResponseData]
    forbidden_record_ids: List[str]
    absent_record_ids: List[str]


class BatchGetRecordResponse(BaseResponse):
    """批量获取记录响应体
    Example:
    ```
    {
      "code": 0,
      "msg": "success",
      "data": {
        "forbidden_record_ids": [
          "recyOaMB2F"
        ],
        "absent_record_ids": [
          "rec111111"
        ],
        "records": [
          {
            "created_by": {
              "avatar_url": "https://internal-api-lark-file.feishu.cn/static-resource/v1/06d568cb-f464-4c2e-bd03-76512c545c5j~?image_size=72x72&amp;cut_type=default-face&amp;quality=&amp;format=jpeg&amp;sticker_format=.webp",
              "email": "",
              "en_name": "Min Zhang",
              "id": "ou_92945f86a98bba075174776959c90eda",
              "name": "张敏"
            },
            "created_time": 1691049973000,
            "fields": {
              "人员": [
                {
                  "avatar_url": "https://internal-api-lark-file.feishu.cn/static-resource/v1/b2-7619-4b8a-b27b-c72d90b06a2j~?image_size=72x72&amp;cut_type=default-face&amp;quality=&amp;format=jpeg&amp;sticker_format=.webp",
                  "email": "minzhang.leben@bytedance.com",
                  "en_name": "Min Zhang",
                  "id": "ou_2910013f1e6456f16a0ce75ede950a0a",
                  "name": "张敏"
                },
                {
                  "avatar_url": "https://internal-api-lark-file.feishu.cn/static-resource/v1/v2_q86-fcb6-4f18-85c7-87ca8881e50j~?image_size=72x72&amp;cut_type=default-face&amp;quality=&amp;format=jpeg&amp;sticker_format=.webp",
                  "email": "minzhang.00@bytedance.com",
                  "en_name": "Min Zhang",
                  "id": "ou_e04138c9633dd0d2ea166d79f548ab5d",
                  "name": "张敏"
                }
              ],
              "修改人": [
                {
                  "avatar_url": "https://internal-api-lark-file.feishu.cn/static-resource/v1/06d568cb-f464-4c2e-bd03-76512c545c5j~?image_size=72x72&amp;cut_type=default-face&amp;quality=&amp;format=jpeg&amp;sticker_format=.webp",
                  "email": "",
                  "en_name": "Min Zhang",
                  "id": "ou_92945f86a98bba075174776959c90eda",
                  "name": "张敏"
                }
              ],
              "创建人": [
                {
                  "avatar_url": "https://internal-api-lark-file.feishu.cn/static-resource/v1/06d568cb-f464-4c2e-bd03-76512c545c5j~?image_size=72x72&amp;cut_type=default-face&amp;quality=&amp;format=jpeg&amp;sticker_format=.webp",
                  "email": "",
                  "en_name": "Min Zhang",
                  "id": "ou_92945f86a98bba075174776959c90eda",
                  "name": "张敏"
                }
              ],
              "创建时间": 1691049973000,
              "单向关联": {
                "link_record_ids": [
                  "recnVYsuqV"
                ]
              },
              "单选": "选项1",
              "双向关联": {
                "link_record_ids": [
                  "recqLvMaXT",
                  "recrdld32q"
                ]
              },
              "地理位置": {
                "address": "东长安街",
                "adname": "东城区",
                "cityname": "北京市",
                "full_address": "天安门广场，北京市东城区东长安街",
                "location": "116.397755,39.903179",
                "name": "天安门广场",
                "pname": "北京市"
              },
              "复选框": true,
              "多行文本": [
                {
                  "text": "多行文本内容1",
                  "type": "text"
                },
                {
                  "mentionNotify": false,
                  "mentionType": "User",
                  "name": "张敏",
                  "text": "@张敏",
                  "token": "ou_2910013f1e6456f16a0ce75ede950a0a",
                  "type": "mention"
                }
              ],
              "多选": [
                "选项1",
                "选项2"
              ],
              "数字": 2323.2323,
              "日期": 1690992000000,
              "最后更新时间": 1702455191000,
              "条码": [
                {
                  "text": "123",
                  "type": "text"
                }
              ],
              "电话号码": "131xxxx6666",
              "自动编号": "17",
              "群组": [
                {
                  "avatar_url": "https://internal-api-lark-file.feishu-boe.cn/static-resource/v1/v2_c8d2cd50-ba29-476f-b7f1-5b5917cb18ej~?image_size=72x72&amp;cut_type=&amp;quality=&amp;format=jpeg&amp;sticker_format=.webp",
                  "id": "oc_cd07f55f14d6f4a4f1b51504e7e97f48",
                  "name": "武侠聊天组"
                }
              ],
              "评分": 3,
              "货币": 1,
              "超链接": {
                "link": "https://bitable.feishu.cn",
                "text": "飞书多维表格官网"
              },
              "进度": 0.66,
              "附件": [
                {
                  "file_token": "Vl3FbVkvnowlgpxpqsAbBrtFcrd",
                  "name": "飞书.jpeg",
                  "size": 32975,
                  "tmp_url": "https://open.feishu.cn/open-apis/drive/v1/medias/batch_get_tmp_download_url?file_tokens=Vl3FbVk11owlgpxpqsAbBrtFcrd&amp;extra={"bitablePerm":{"tableId":"tblBJyX6jZteblYv","rev":90}}",
                  "type": "image/jpeg",
                  "url": "https://open.feishu.cn/open-apis/drive/v1/medias/Vl3FbVk11owlgpxpqsAbBrtFcrd/download?extra={"bitablePerm":{"tableId":"tblBJyX6jZteblYv","rev":90}}"
                }
              ]
            },
            "last_modified_by": {
              "avatar_url": "https://internal-api-lark-file.feishu.cn/static-resource/v1/06d568cb-f464-4c2e-bd03-76512c545c5j~?image_size=72x72&amp;cut_type=default-face&amp;quality=&amp;format=jpeg&amp;sticker_format=.webp",
              "email": "",
              "en_name": "Min Zhang",
              "id": "ou_92945f86a98bba075174776959c90eda",
              "name": "张敏"
            },
            "last_modified_time": 1702455191000,
            "record_id": "recyOaMB2F",
            "shared_url": "https://example.feishu.cn/record/KBcNrNtpWePAlscCvdmb6ZcSc5b"
          }
        ]
      }
    }
    ```"""

    data: BatchGetRecordResponseData


class BatchUpdateRecordResponse(BatchCreateRecordResponse):
    pass



from typing import Dict, List, Union

from .._common import BaseModel


class CallbackOperator(BaseModel):
    tenant_key: Union[str, None] = None
    user_id: Union[str, None] = None
    open_id: Union[str, None] = None


class CallbackAction(BaseModel):
    value: Union[Dict, str]
    tag: str
    timezone: str
    name: str
    form_value: Union[Dict, None] = None
    input_value: Union[str, None] = None
    option: Union[str, None] = None
    options: Union[List[str], None] = None
    checked: Union[bool, None] = None


class CallbackContext(BaseModel):
    url: Union[str, None] = None
    preview_token: Union[str, None] = None
    open_message_id: str
    open_chat_id: str


class CallbackEvent(BaseModel):
    operator: CallbackOperator
    token: str
    action: CallbackAction
    host: str
    delivery_type: Union[str, None] = None
    context: CallbackContext
